/*
  Originally developped for the SmbShareScan project.
  Copyright (C) 2004-2005  Guillaume Roche

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <libsmbclient.h>
#include <errno.h>
#include <pthread.h>
#include <regex.h>
#include "bsd_sys_queue.h"
#include "samba.h"
#include "interface.h"
//#include "system.h"
//#include "savexml.h"

extern int optind;
extern char * optarg;

extern struct samba_state samba;

int ips = 0;

struct gtk_data gtk;


struct result {
	struct fileshare * result;

	STAILQ_ENTRY( result ) result_next;
};

STAILQ_HEAD( slist_result, result ) result_list;


void Loading(GtkWidget* widget, gpointer data);
void scanNetwork(GtkProgressBar* pWidget);
GtkWidget* about ();
void addDir(GtkTreeStore *pTreeStore, GtkTreeIter pIter1, struct fileshare *);
void addFile(GtkTreeStore *pTreeStore, GtkTreeIter pIter1, struct fileshare *);
void addTreeBrowse();
GtkWidget* browse ();
int search_file(GtkWidget *widget, void *);
void addListSearch();
GtkWidget* search ();
GtkWidget* my_tasks ();
void OnAbout(GtkWidget* widget, gpointer data);
void quitter(GtkWidget* widget, gpointer data);
void OnDestroy(GtkWidget *pWidget, gpointer pData);


void Loading(GtkWidget* widget, gpointer data)
{
  GtkWidget *pProgressBox;
  GtkWidget *pProgress;
  GtkWidget *pLabel;
  
  pProgressBox = gtk_dialog_new_with_buttons("Scanning Progress ...",
                                             GTK_WINDOW(data),
                                             GTK_DIALOG_MODAL,
                                             GTK_STOCK_OK,GTK_RESPONSE_OK,
                                             GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
                                             NULL);
  pLabel = gtk_label_new("Would you like to scan the network ?\n");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pProgressBox)->vbox), pLabel, TRUE, FALSE, 0);
  pProgress = gtk_progress_bar_new();
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pProgressBox)->vbox), pProgress, TRUE, FALSE, 0);

  gtk_widget_show_all(GTK_DIALOG(pProgressBox)->vbox);
  
  /* On lance la boite de dialogue et on recupere la reponse */
  switch (gtk_dialog_run(GTK_DIALOG(pProgressBox)))
    {
      /* L utilisateur valide */
    case GTK_RESPONSE_OK:
      scanNetwork(GTK_PROGRESS_BAR(pProgress));
      break;
      /* L utilisateur annule */
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_NONE:
    default:
      break;
    }
  gtk_widget_destroy(pProgressBox);
  
}

void refresh_status() {
  char status[BUFSIZ];
  unsigned int thread_num = 0;
  unsigned long file_num = 0;

  pthread_mutex_lock( &samba.smb_thread_mutex );
  thread_num = samba.smb_threads;
  file_num = samba.smb_files;
  pthread_mutex_unlock( &samba.smb_thread_mutex );

  while (thread_num != 0) {
    snprintf(status, sizeof( status ), "Scanned %lu files scanned         Nb Threads : %u", 
		    file_num, thread_num);
    gtk_statusbar_push (GTK_STATUSBAR (gtk.pStatusBar), GPOINTER_TO_INT(gtk.iContextId), status);

    //new_tree_store();
    //addTreeBrowse(GTK_TREE_MODEL(pTreeStore));
    //sleep(10);
    usleep(50000);

    pthread_mutex_lock( &samba.smb_thread_mutex );
    thread_num = samba.smb_threads;
    file_num = samba.smb_files;
    pthread_mutex_unlock( &samba.smb_thread_mutex );
  }
  addTreeBrowse(GTK_TREE_MODEL(gtk.pTreeStore));
}


void scanNetwork(GtkProgressBar* pWidget) {
  /* Initialisation */
  struct network * net = NULL;
  unsigned int thread_num = 0;
  unsigned long file_num = 0;
  gdouble dFraction;
  gint i, j;
  gint iTotal = 100;
  char status[BUFSIZ];

  pthread_t thread;
  void *retour;

  /* free all things before */

  /* free tree */
  
  net = STAILQ_FIRST( &samba.smb_network );
  if( net == NULL ) {
    net = network_alloc( );
    STAILQ_INSERT_TAIL( &samba.smb_network, net, network_next );
    samba.smb_network_size ++;
  }

  printf("System: Start scanning\n");
  
  gtk_statusbar_push (GTK_STATUSBAR (gtk.pStatusBar), GPOINTER_TO_INT(gtk.iContextId), "Scanning ...");
  if (ips == 1) {
    if (pthread_create (&thread, NULL, scan_ips, net) < 0) {
      fprintf (stderr, "Impossible de crer le thread\n");
      exit (1);
    }
  } else if (ips == 0) {
      if (pthread_create (&thread, NULL, scan_bm, net) < 0) {
        fprintf (stderr, "Impossible de crer le thread\n");
        exit (1);
      }
    }

  pthread_mutex_lock( &samba.smb_thread_mutex );
  samba.smb_threads++;
  printf("samba.smb_threads : %d\n", samba.smb_threads );
  pthread_mutex_unlock( &samba.smb_thread_mutex );

  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pWidget), 0.0);

  gtk_grab_add(GTK_WIDGET(pWidget));
  j=0;

  pthread_mutex_lock( &samba.smb_thread_mutex );
  thread_num = samba.smb_threads;
  file_num = samba.smb_files;
  pthread_mutex_unlock( &samba.smb_thread_mutex );
	
  while (thread_num > 0 ) {
    for(i = 0 ; i < iTotal ; ++i)
      {
        dFraction = (gdouble)i / (gdouble)iTotal;
        
        usleep(50000);

        pthread_mutex_lock( &samba.smb_thread_mutex );
        thread_num = samba.smb_threads;
        file_num = samba.smb_files;
        pthread_mutex_unlock( &samba.smb_thread_mutex );
         
        /*Change the status bar*/
        snprintf(status, sizeof( status ), "Scanned %lu files scanned         Nb Threads : %u", 
		      file_num, thread_num);
        //sprintf(status,"Scanning ... %d files scanned", samba.smb_files);
        gtk_statusbar_push (GTK_STATUSBAR (gtk.pStatusBar), GPOINTER_TO_INT(gtk.iContextId), status);
        /* Modification de la valeur de la barre de progression */
        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pWidget), dFraction);
         
        /* On donne la main a GTK+ */
        gtk_main_iteration ();
      }
    if ((j%2) == 0) 
      gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(pWidget),GTK_PROGRESS_RIGHT_TO_LEFT);
    else
      gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(pWidget),GTK_PROGRESS_LEFT_TO_RIGHT);
    j++;

  }

  /* On supprime le grab sur la barre de progression */
  gtk_grab_remove(GTK_WIDGET(pWidget));
   
  printf("System: wait for thread\n");
  (void)pthread_join (thread, &retour);
  printf("System: thread finished\n");
  printf("System: nbfiles : %lu\n", samba.smb_files);
  
  /*if (pthread_create (&thread2, NULL, refresh_status, NULL) < 0) {
    fprintf (stderr, "Impossible de crer le thread\\n");
    exit (1);
    }*/

  snprintf(status, sizeof(status), "Scanned %lu files scanned         Nb Threads : %u", 
		  samba.smb_files, samba.smb_threads);
  gtk_statusbar_push (GTK_STATUSBAR (gtk.pStatusBar), GPOINTER_TO_INT(gtk.iContextId), status);

  /*while (samba.smb_threads != 0) {
    sprintf(status,"Scanned %d files scanned         Nb Threads : %d", samba.smb_files, samba.smb_threads);
    gtk_statusbar_push (GTK_STATUSBAR (pStatusBar), GPOINTER_TO_INT(iContextId), status);
    }*/

  addTreeBrowse(GTK_TREE_MODEL(gtk.pTreeStore));
}


/* A Propos */
GtkWidget* about () { 
  GtkWidget *label, *name, *thanksto, *email, *file_url;
  GtkWidget *hbox1, *hbox2;
  GtkWidget *vbox, *vbox1, *vbox2;
  GtkWidget *image;
  
  image = gtk_image_new_from_file ("logo/SmbShareScan.png");
  label = gtk_label_new ((" v" PROG_VERSION));
  name = gtk_label_new ((" Guillaume Roche"));
  thanksto = gtk_label_new ((" Thanks to Aurelien Jacobs and Jean-Philippe Andriot "));
  email = gtk_label_new ((" Email : guigeek@gmail.com "));
  file_url= gtk_label_new ((" Web : smbsharescan.free.fr "));

  vbox = gtk_vbox_new (FALSE, 0);
  vbox1 = gtk_vbox_new (FALSE, 0);
  vbox2 = gtk_vbox_new (FALSE, 0);
  hbox1 = gtk_hbox_new (FALSE, 0);
  hbox2 = gtk_hbox_new (FALSE, 0);

  //gtk_box_pack_start ( GTK_BOX (vbox2), label, TRUE, TRUE, 0);
  gtk_box_pack_start ( GTK_BOX (vbox1), image, TRUE, TRUE, 0);
  gtk_box_pack_start ( GTK_BOX (vbox1), label, TRUE, TRUE, 0);	
  gtk_box_pack_start ( GTK_BOX (vbox2), name, TRUE, TRUE, 0);
  gtk_box_pack_start ( GTK_BOX (vbox2), thanksto, TRUE, TRUE, 0);
  gtk_box_pack_start ( GTK_BOX (vbox2), email, TRUE, TRUE, 0);
  gtk_box_pack_start ( GTK_BOX (vbox2), file_url, TRUE, TRUE, 0);
  
  gtk_box_pack_start ( GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);
  gtk_box_pack_start ( GTK_BOX (hbox2), vbox2, TRUE, TRUE, 0);

  gtk_box_pack_start ( GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
  gtk_box_pack_start ( GTK_BOX (vbox), hbox2, TRUE, TRUE, 0);
  return vbox;
}


/*static gint button_press (GtkWidget *widget, GdkEvent *event)
{
  if (event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bevent = (GdkEventButton *) event;
    gtk_menu_popup (GTK_MENU(widget), NULL, NULL, NULL, NULL,
                    bevent->button, bevent->time);
                    // On indique  l'appelant que l'on a gr cet vnement. 
    return TRUE;
  }
  // On indique  l'appelant que l'on n'a pas gr cet vnement.
  return FALSE;
}*/

int smbshare_gtk_add_file( GtkTreeStore * pTree, GtkTreeIter * pIterLast, const char* name )
{  
  GtkTreeIter pIter; 
  
  enum {
    BMP_COLUMN,
    TEXT_COLUMN,
    //SIZE_COLUMN,
    N_COLUMN
  };
  
  gtk_tree_store_append(pTree, &pIter, pIterLast);
  gtk_tree_store_set(pTree, &pIter,
                     BMP_COLUMN, gtk.gtk_pix_file,
                     TEXT_COLUMN, name,
                     //SIZE_COLUMN, (file->len),
                     -1);
  return 0;
}

int smbshare_gtk_add_dir( GtkTreeStore * pTree, GtkTreeIter * pIterLast, const char* name, GtkTreeIter * pIterOut )
{  
  enum {
    BMP_COLUMN,
    TEXT_COLUMN,
    //SIZE_COLUMN,
    N_COLUMN
  };

  gtk_tree_store_append(pTree, pIterOut, pIterLast);
  gtk_tree_store_set(pTree, pIterOut,
                     BMP_COLUMN, gtk.gtk_pix_dir,
                     TEXT_COLUMN, name,
                     //SIZE_COLUMN, (file->len),
                     -1);
  return 0;
}



int smbshare_add_rec_share(GtkTreeStore * pTree, GtkTreeIter * pIterRoot, struct fileshare * share_root )
{
  struct fileshare * share;

  STAILQ_FOREACH( share, &share_root->share_list, share_next ) {
    if ( share->share_type == SMBC_FILE ) { 

      smbshare_gtk_add_file( pTree, pIterRoot, share->share_name );

    } else {
      GtkTreeIter pIterSub;

      /* add directory to tree */
      smbshare_gtk_add_dir( pTree, pIterRoot, share->share_name, &pIterSub );

      smbshare_add_rec_share( pTree, &pIterSub, share ); 
    }
  }
  return 0;
}

void addTreeBrowse() {
  struct computer *computer = NULL;
  struct fileshare * share = NULL;
  struct workgroup *work = NULL;
  struct network *net = NULL;
  GtkTreeIter pIterComputers, pIterNetworks;
  GtkTreeIter pIterFiles;
  enum {
    BMP_COLUMN,
    TEXT_COLUMN,
    SIZE_COLUMN,
    N_COLUMN
  };
  
  
  net = STAILQ_FIRST( &samba.smb_network );
  if ( net == NULL )
	  return ;

  STAILQ_FOREACH( work, &net->network_workgroup, workgroup_next )
  {
    gtk_tree_store_append(gtk.pTreeStore, &pIterNetworks, NULL);
    gtk_tree_store_set(gtk.pTreeStore, &pIterNetworks,
                       BMP_COLUMN, gtk.gtk_pix_network,
                       TEXT_COLUMN, work->workgroup_name,
                       -1);
    /* Insertion des elements */
    STAILQ_FOREACH( computer, &work->workgroup_computer, computer_next )
    {
      /* Creation de la nouvelle ligne */
      gtk_tree_store_append(gtk.pTreeStore, &pIterComputers, &pIterNetworks);
      /* Mise a jour des donnees */
      gtk_tree_store_set(gtk.pTreeStore, &pIterComputers,
                         BMP_COLUMN, gtk.gtk_pix_computer,
                         TEXT_COLUMN, computer->computer_netbiosname,
                         -1);

      //gtk_signal_connect_object(GTK_OBJECT(button), "event",GTK_SIGNAL_FUNC (button_press), GTK_OBJECT(menu));
      STAILQ_FOREACH( share, &computer->computer_share, share_next) 
      {
        /* add root shared directory */
        gtk_tree_store_append(gtk.pTreeStore, &pIterFiles, &pIterComputers);

        gtk_tree_store_set(gtk.pTreeStore, &pIterFiles,
                           BMP_COLUMN, gtk.gtk_pix_dir,
                           TEXT_COLUMN, share->share_name,
                           -1);

	    /* do recursive search */

        smbshare_add_rec_share( gtk.pTreeStore, &pIterFiles, share );
      }
    }
  } 
}

void essai(GtkTreeSelection *treeselection, gpointer user_data) {
  printf("Changed\n");
}


void new_tree_store() {
  GtkWidget *pListView;

  enum {
    BMP_COLUMN,
    TEXT_COLUMN,
    //SIZE_COLUMN,
    N_COLUMN
  };
  gtk.pTreeStore = gtk_tree_store_new(N_COLUMN, GDK_TYPE_PIXBUF, G_TYPE_STRING);//, G_TYPE_INT);
  pListView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(gtk.pTreeStore));
  gtk.selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(pListView));
  gtk_tree_selection_set_mode(gtk.selection, GTK_SELECTION_BROWSE);
  g_signal_connect(G_OBJECT(gtk.selection), "changed", G_CALLBACK(essai), NULL);
}



void open_with(GtkWidget* widget, gpointer data)
{
  GtkWidget *popen;
  GtkWidget *pLabel;
  
  popen = gtk_dialog_new_with_buttons("Opening  ...",
                                      GTK_WINDOW(data),
                                      GTK_DIALOG_MODAL,
                                      GTK_STOCK_OK,GTK_RESPONSE_OK,
                                      GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
                                      NULL);

  /* hbox */
  pLabel = gtk_label_new("Open with ...\n");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popen)->vbox), pLabel, TRUE, FALSE, 0);
  
  /* textfield in hbox */

  gtk_widget_show_all(GTK_DIALOG(popen)->vbox);
  
  /* On lance la boite de dialogue et on recupere la reponse */
  switch (gtk_dialog_run(GTK_DIALOG(popen)))
    {
      /* L utilisateur valide */
    case GTK_RESPONSE_OK:
      // Open with libsmbclient ...
      break;
      /* L utilisateur annule */
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_NONE:
    default:
      break;
    }
  gtk_widget_destroy(popen);
  
}




void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
{
  GtkWidget *popup;
  
  gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL, 3, t);
  printf ("**** PASS 1\n");
}

/* Affiche une chane lorsqu'un item de menu est choisi. */
static void menuitem_response (GtkWidget *widget, gchar *string)
{
  //int fd;
  if(strcmp(string,"Open") == 0) {
    printf("%s\n", string);
  }
  if(strcmp(string,"Save as") == 0) {
    printf("%s\n", string);
    
  }
}


int treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
  enum {
    BMP_COLUMN,
    TEXT_COLUMN,
    SIZE_COLUMN,
    N_COLUMN
  };
  GtkTreeIter iter, iter2, iter3, iter4, iter5;
  GtkTreeSelection *selection = NULL;
  GtkTreeModel *treemodel = NULL;
  GtkWidget *pListView = NULL;

  GtkWidget *menu_items = NULL;
  GtkWidget *menu = NULL;
  //tree_entry_t *en;
  GtkSelectionMode mode = 0;
  //GtkTreePath *treepath = NULL;
  gchar *newpath = NULL;
  newpath = "";
  gboolean has_selection;
  gint nb;
  
  
  //GtkTreePath *selectpath;
  //GtkTreeView *pListView = (GtkTreeView *) data;
 
  pListView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(gtk.pTreeStore));
  if (pListView == NULL) {
    printf("error gtk_tree_view_new_with_model\n");
    return 0;
  }

  /*
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(pListView));
  if (selection == NULL) {
    printf("error gtk_tree_view_get_selection\n");
    return 0;
  }
  */

  //GtkTreeModel *treemodel = GTK_TREE_MODEL(pTreeStore);
  treemodel = gtk_tree_view_get_model(GTK_TREE_VIEW(pListView));
  if (treemodel == NULL) {
    printf("error gtk_tree_view_get_model\n");
    return 0;
  }
  //tree_details_t *tree_details = get_tree_details(treeview);
  if (event->button != 3) {
	  return 0;
  }

  printf("System: begin looking for item\n");
  if ((gtk_tree_model_get_iter_first(treemodel, &iter2)) == TRUE) {
    newpath = gtk_tree_model_get_string_from_iter(treemodel, &iter2);

    printf ("path essai : %s\n", newpath);
    if ((gtk_tree_model_iter_children(treemodel, &iter3, &iter2)) == TRUE) {
     if ((gtk_tree_model_iter_children(treemodel, &iter4, &iter3)) == TRUE) {
        newpath = gtk_tree_model_get_string_from_iter(treemodel, &iter4);
        
        printf ("path essai : %s\n",newpath);
        if ((gtk_tree_model_iter_children(treemodel, &iter5, &iter4)) == TRUE) {
          printf("cool\n");
          newpath = gtk_tree_model_get_string_from_iter(treemodel, &iter5);
          printf ("path essai : %s\n",newpath);
          gtk_tree_model_get(treemodel, &iter5, TEXT_COLUMN, &newpath, -1);
          printf ("path truc : %s\n",newpath);
          nb = gtk_tree_model_iter_n_children  (treemodel, &iter4);
          printf ("nb : %d\n",nb);
        }
      }
    }
  }

  //selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(pListView));
  if (selection == NULL) {
    printf("System error : no selection ? error gtk_tree_view_get_selection\n");
    return 0;
  }
    
  mode = gtk_tree_selection_get_mode (selection);
  printf ("**** PASS 3 %d\n", mode);

  g_print ("list = %p\n", gtk_tree_selection_get_selected_rows(selection, &treemodel));
  has_selection = gtk_tree_selection_get_selected(selection, &treemodel, &iter);
  printf("TRUE OR FALSE : %d iter : %d\n", has_selection, iter.stamp);
  if (has_selection==TRUE)
  {
      printf ("**** PASS 2\n");
      //bool2 = gtk_tree_selection_iter_is_selected(selection, &iter);
      //printf("boolean 2 : %d\n",bool2);
      newpath = gtk_tree_model_get_string_from_iter(treemodel, &iter);
      printf ("path essai : %s\n",newpath);
      gtk_tree_model_get(treemodel, &iter, TEXT_COLUMN, &newpath, -1);
      printf ("path truc : %s\n",newpath);
      //gtk_tree_model_get(treemodel, &iter, TEXT_COLUMN, &newpath, -1);
      //printf ("path : %s\n",newpath);
  }

  menu = gtk_menu_new();
  
  /* Cration d'un nouveau item de menu avec un nom... */
  menu_items = gtk_menu_item_new_with_label("Open");
  /* ...et ajout de celui-ci dans le menu. */
  gtk_menu_append(GTK_MENU (menu), menu_items);
  /* On fait quelque chose d'intressant lorsque l'item est
   * slectionn. */
  gtk_signal_connect (GTK_OBJECT(menu_items), "activate",
                      GTK_SIGNAL_FUNC(menuitem_response), (gpointer)
                      g_strdup("Open"));                    
  /* Affichage du widget. */
  gtk_widget_show(menu_items);
    
    
  /* Cration d'un nouveau item de menu avec un nom... */
  menu_items = gtk_menu_item_new_with_label("Save as");
  /* ...et ajout de celui-ci dans le menu. */
  gtk_menu_append(GTK_MENU (menu), menu_items);
  /* On fait quelque chose d'intressant lorsque l'item est
   * slectionn. */
  gtk_signal_connect (GTK_OBJECT(menu_items), "activate",
                      GTK_SIGNAL_FUNC(menuitem_response), (gpointer)
                      g_strdup("Save as"));                    
  /* Affichage du widget. */
  gtk_widget_show(menu_items);

    
  GdkEventButton *bevent = (GdkEventButton *) event;
  gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
                  bevent->button, bevent->time);
  return 1;
  
}




GtkWidget* browse (Void)
{ 
  GtkWidget *vbox;
  GtkWidget *pTreeView;
  GtkWidget *pScrollbar;
  //GtkTreeStore *pTreeStore;
  GtkTreeViewColumn *pColumn;
  GtkCellRenderer *pCellRenderer;
  enum {
    BMP_COLUMN,
    TEXT_COLUMN,
    SIZE_COLUMN,
    N_COLUMN
  };
  //net = read_xml("galilee.xml");

  vbox = gtk_vbox_new (FALSE, 0);
  /* Creation du modele */
  new_tree_store();
  //pTreeStore = gtk_tree_store_new(N_COLUMN, GDK_TYPE_PIXBUF, G_TYPE_STRING);
  
  /* Creation de la vue */
  pTreeView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(gtk.pTreeStore));
  g_signal_connect(pTreeView, "button-press-event", G_CALLBACK(treeclick), pTreeView);
  /* Ajouter arbre */
  addTreeBrowse();
  
  /* Creation de la premiere colonne */
  pCellRenderer = gtk_cell_renderer_pixbuf_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Icon",
                                                     pCellRenderer,
                                                     "pixbuf", BMP_COLUMN,
                                                     NULL);

  //g_signal_connect(G_OBJECT(column), "clicked", G_CALLBACK(titles_off), (gpointer) ptreeview);

  /* Ajout de la colonne a la vue */
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);

  /* Creation de la deuxieme colonne */
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("Name",
                                                     pCellRenderer,
                                                     "text", TEXT_COLUMN,
                                                     NULL);

  /* Ajout de la colonne a la vue */
  gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);


  /* Creation de la deuxieme colonne */
  //pCellRenderer = gtk_cell_renderer_text_new();
  /*pColumn = gtk_tree_view_column_new_with_attributes("Size",
                                                     pCellRenderer,
                                                     "text", SIZE_COLUMN,
                                                     NULL);*/

  /* Ajout de la colonne a la vue */
  //gtk_tree_view_append_column(GTK_TREE_VIEW(pTreeView), pColumn);


  /* Ajout de la vue a la fenetre */
  pScrollbar = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pScrollbar),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(pScrollbar), pTreeView);
  gtk_box_pack_start(GTK_BOX(vbox), pScrollbar, TRUE, TRUE, 0);
  return vbox;
}



int smbshare_match_regex( const regex_t * reg, const char * line, const size_t line_len ) {
	regmatch_t match;
	int ret;

	ret = regexec( reg, line, 1, &match, REG_NOTBOL | REG_NOTEOL);
	if ( ret != 0 /* || match.rm_so != 0 || match.rm_eo != (int)line_len */ )
		return 0;

	return 1;
}





int smbshare_search_rec_share(regex_t * reg, struct fileshare * share_root )
{
  struct fileshare * share;
  struct result * result;

  STAILQ_FOREACH( share, &share_root->share_list, share_next ) {
    if ( share->share_type == SMBC_FILE ) { 

      /* check file name */
      if( smbshare_match_regex( reg, share->share_name, strlen( share->share_name )) ) {
        /* good result ! */
        result = (struct result*)Malloc(sizeof(struct result));
	result->result = share;
        STAILQ_INSERT_TAIL( &result_list, result, result_next );
      }

    } else {
      /* check dir name */
      if( smbshare_match_regex( reg, share->share_name, strlen( share->share_name )) ) {
        /* good result ! */
        result = (struct result*)Malloc(sizeof(struct result));
	result->result = share;
        STAILQ_INSERT_TAIL( &result_list, result, result_next );
      }

      /* do rec search */
      smbshare_search_rec_share( reg, share ); 
    }
  }
  return 0;
}

int search_file(GtkWidget *widget, void *data) {
  struct workgroup * work;
  struct network * net;
  struct computer * computer;
  struct fileshare * share;
  struct result * result;
  //char title[100];
  const gchar *title;
  regex_t parse_regex;
  int ret;

  while ( (result = STAILQ_FIRST( &result_list )) ) {
     
    STAILQ_REMOVE_HEAD( &result_list, result_next );

    free( result );
  }
  
  title = gtk_entry_get_text(GTK_ENTRY(gtk.entrySearch));

  printf("Search file : %s\n", title);

  net = STAILQ_FIRST( &samba.smb_network );
  if ( net == NULL ) {
    printf("System: no network: no search to do\n");
    return FALSE;
  }

  /* clear search list */
  if ( !STAILQ_EMPTY( &result_list ) ) {
    printf("System: result list not empty\n");
    return FALSE;
  }

  /* build regex from title */
  ret = regcomp( &parse_regex, title, REG_EXTENDED ) ;
  if ( ret != 0 ) {
    printf("System: not a valid regular expression\n");
    return FALSE;
  }

  STAILQ_FOREACH( work, &net->network_workgroup, workgroup_next )
  {

    STAILQ_FOREACH( computer, &work->workgroup_computer, computer_next )
    {

      STAILQ_FOREACH( share, &computer->computer_share, share_next) 
      {
        /* do recursive search */

        smbshare_search_rec_share( &parse_regex, share );

      }
    }
  }

  //*(dat->result) = (smbcfile**)malloc(20 * sizeof(char*));
  /*for (i=0;i<20;i++) {
    (*(dat->result))[i] = (smbcfile*)malloc(256 * sizeof(char));
    init_file((*(dat->result))[i]);
    }*/
  char * dir_url = NULL;
  size_t len = 0;
  printf("Search results: \n" );
  STAILQ_FOREACH( result, &result_list, result_next ) {
	if (result->result->share_type == SMBC_DIR) {
		len = strlen(result->result->share_url) + strlen(result->result->share_name) + 8;	/* +8 because of : 'smb://' and '/' and '\0'  */
    	dir_url = (char*)Malloc( len ); 
    	snprintf( dir_url, len, "smb://%s/%s", result->result->share_computer->computer_netbiosname, result->result->share_url);
    	printf(" + %s\n", result->result->share_name );
		printf("\t - %s\n", dir_url );
	}
	else if (result->result->share_type == SMBC_FILE) {
		printf(" + %s\n", result->result->share_name );
		printf("\t - %s\n", result->result->share_url );
	}
  }

  regfree( &parse_regex );
  
  printf("System: FIN search file \n");
  addListSearch();
  return FALSE;	/* needed by GTK */
}


void addListSearch() {
  GtkTreeIter pIter;
  struct result * result;
  enum {
    NAME_COLUMN,
    URL_COLUMN,
    N_COLUMN
  };
  /* Efface la recherche prcdente */
  gtk_list_store_clear (gtk.pListStore);
  
  /* Insertion des elements */
  STAILQ_FOREACH( result, &result_list, result_next ) {
      
	  // Creation de la nouvelle ligne
      gtk_list_store_append(gtk.pListStore, &pIter);

      // Mise a jour des donnees
	  char * dir_url = NULL;
      size_t len = 0;
	  if (result->result->share_type == SMBC_DIR) {
		len = strlen(result->result->share_url) + strlen(result->result->share_name) + 8;	/* +8 because of : 'smb://' and '/' and '\0'  */
    	dir_url = (char*)Malloc( len ); 
    	snprintf( dir_url, len, "smb://%s/%s", result->result->share_computer->computer_netbiosname, result->result->share_url);
      	gtk_list_store_set(gtk.pListStore, &pIter,
                         NAME_COLUMN, result->result->share_name,
                         URL_COLUMN, dir_url,
                         -1);
	  	}
	  else if (result->result->share_type == SMBC_FILE) {
		gtk_list_store_set(gtk.pListStore, &pIter,
                         NAME_COLUMN, result->result->share_name,
                         URL_COLUMN, result->result->share_url,
                         -1);	
		}	
   }
}


GtkWidget* search (Void)
{ 
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *labSearch, *buttSearch;
  GtkWidget *pListView;
  GtkWidget *pScrollbar;
  //GtkListStore *pListStore;
  GtkTreeViewColumn *pColumn;
  GtkCellRenderer *pCellRenderer;
  //gint i;
  enum {
    NAME_COLUMN,
    URL_COLUMN,
    N_COLUMN
  };
  
    
  /* Create a vBox */
  vbox = gtk_vbox_new (FALSE, 0);

  /* Create a hBox */
  hbox = gtk_hbox_new (FALSE, 0);
  labSearch = gtk_label_new("Filename :  ");
  gtk_box_pack_start(GTK_BOX(hbox), labSearch, FALSE, FALSE, 0);

  gtk.entrySearch = gtk_entry_new();
  gtk_signal_connect( GTK_OBJECT(gtk.entrySearch), "activate", GTK_SIGNAL_FUNC(search_file), /* XXX: data */ NULL);
  gtk_box_pack_start(GTK_BOX(hbox), gtk.entrySearch, TRUE, TRUE, 0);

  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  buttSearch = gtk_button_new_from_stock(GTK_STOCK_FIND);
  gtk_box_pack_start(GTK_BOX(vbox), buttSearch, FALSE, TRUE, 0);
  g_signal_connect(G_OBJECT(buttSearch), "clicked", G_CALLBACK(search_file), /* XXX: data */ NULL );


  /* Creation du modele */
  //pListStore = gtk_list_store_new(N_COLUMN, G_TYPE_STRING, G_TYPE_INT);
  gtk.pListStore = gtk_list_store_new(N_COLUMN, G_TYPE_STRING, G_TYPE_STRING);
  /* Creation de la vue */
  pListView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(gtk.pListStore));
  
  /* Insertion des elements */
  /*for(i = 0 ; i < 1 ; ++i)  
    {
      GtkTreeIter pIter;
      
      // Creation de la nouvelle ligne
      gtk_list_store_append(pListStore, &pIter);
      
      // Mise a jour des donnees
      gtk_list_store_set(pListStore, &pIter,
                         NAME_COLUMN, (((*dat.result)[i])->name),
                         URL_COLUMN, (((*dat.result)[i])->file_url),
                         -1);
                         }*/
    
   
  /* Creation de la premiere colonne */
  pCellRenderer = gtk_cell_renderer_text_new();
  pColumn = gtk_tree_view_column_new_with_attributes("FileName",
                                                     pCellRenderer,
                                                     "text", NAME_COLUMN,
                                                     NULL);
  /* Ajout de la colonne a la vue */
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);

  /*Creation de la deuxieme colonne */
  //pCellRenderer = gtk_cell_renderer_toggle_new();
  pCellRenderer = gtk_cell_renderer_text_new();
   /*pColumn = gtk_tree_view_column_new_with_attributes("CheckBox",
                                                      pCellRenderer,
                                                      "active", TOGGLE_COLUMN,
                                                      NULL);*/
   pColumn = gtk_tree_view_column_new_with_attributes("Url",
                                                      pCellRenderer,
                                                      "text", URL_COLUMN,
                                                      NULL);
  /* Ajout de la colonne  la vue */ 
  gtk_tree_view_append_column(GTK_TREE_VIEW(pListView), pColumn);
 
  /* Ajout de la vue a la fenetre */
  pScrollbar = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pScrollbar),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(pScrollbar), pListView);
  
  gtk_box_pack_start(GTK_BOX(vbox), pScrollbar, TRUE, TRUE, 0);
  return vbox;
}

GtkWidget* my_tasks (Void)
{ 
        return NULL;
}

void openXML() {
  //net = read_xml("galilee.xml");
  
  struct network * net = STAILQ_FIRST( &samba.smb_network );
  
  printf("size of network : %d\n",   net->network_size);
/*
  printf("size of workgroup : %d\n", net->network_workgroup[0]->workgroup_size);
  printf("size of computer : %d\n",  net->network_workgroup[0]->workgroup_computers[0]->computer_share_size);
*/
  addTreeBrowse();
}

static int parse_args( int argc, char ** argv )
{
  int ch;

  while ((ch = getopt(argc, argv, "sivt:l:")) != -1)
    switch (ch) {
      case 'i':
        ips = 1;
        break;
      case 't':
	printf("SmbShare: nothing just a test <%s>\n", optarg );
        break;
      case 'l':
	samba.smb_error_lvl = atoi(optarg);
	break;
      case 'v':
	printf("%s v%s\n", PROG_NAME, PROG_VERSION );
      case '?':
      default:
	return -1;
    }
  argc -= optind;
  argv += optind;

  return 0;
}

int main(int argc, char **argv)
{
  static GtkWidget *pWindow;
  GtkWidget *notebook;
  GtkWidget *vbox;
  GtkWidget *pMenuBar;
  GtkWidget *pMenu;
  GtkWidget *pMenuItem;

  memset( &samba, 0, sizeof( struct samba_state ));

  gtk_init(&argc, &argv);

  if ( parse_args( argc, argv ) != 0 ) {
    return EXIT_SUCCESS;
  }

  if( init_samba( ) < 0 ) {
    printf("SmbShare : unable to init samba\n");
    return EXIT_FAILURE;
  }
  
  /* GTK DATA INIT */

  STAILQ_INIT (&result_list) ;
  memset ( &gtk, 0, sizeof ( struct gtk_data ));
  
  /* Chargement des images */
  gtk.gtk_pix_network  = gdk_pixbuf_new_from_file("pixmaps/network_local.png", NULL);
  gtk.gtk_pix_computer = gdk_pixbuf_new_from_file("pixmaps/screen_black.png", NULL);
  gtk.gtk_pix_dir      = gdk_pixbuf_new_from_file("pixmaps/folder_yellow.png", NULL);
  gtk.gtk_pix_file     = gdk_pixbuf_new_from_file("pixmaps/iconfile.png", NULL);

  /* WINDOW CREATION */

  /* Creation de la fenetre */
  pWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  /* Definition de la position */
  gtk_window_set_position(GTK_WINDOW(pWindow), GTK_WIN_POS_CENTER);
  /* Definition de la taille de la fenetre */
  gtk_window_set_default_size(GTK_WINDOW(pWindow), 800, 600);
  /* Titre de la fenetre */
  gtk_window_set_title(GTK_WINDOW(pWindow), "SmbShareScan");

  /* Connexion du signal "destroy" */
  g_signal_connect(G_OBJECT(pWindow), "destroy", G_CALLBACK(gtk_main_quit), NULL);
  
  /* Creation d'un vbox*/
  vbox=gtk_vbox_new(FALSE,5);
  gtk_container_add(GTK_CONTAINER(pWindow),vbox);

  /**** Creation du menu ****/
  /* ETAPE 1 */
  pMenuBar = gtk_menu_bar_new();
  /** Premier sous-menu **/
  /* ETAPE 2 */
  pMenu = gtk_menu_new();
  /* ETAPE 3 */
  pMenuItem = gtk_menu_item_new_with_label("Scan");
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(Loading), NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  
  pMenuItem = gtk_menu_item_new_with_label("Open XML Save");
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(openXML), NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);

  //pMenuItem = gtk_menu_item_new_with_label("Quit");
  pMenuItem = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT,NULL);
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(quitter), (GtkWidget*) pWindow);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  /* ETAPE 4 */
  pMenuItem = gtk_menu_item_new_with_label("Network");
  /* ETAPE 5 */
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(pMenuItem), pMenu);
  /* ETAPE 6 */
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenuBar), pMenuItem);

  /** Second sous-menu **/
  /* ETAPE 2 */
  pMenu = gtk_menu_new();
  /* ETAPE 3 */
  pMenuItem = gtk_menu_item_new_with_label("About");
  g_signal_connect(G_OBJECT(pMenuItem), "activate", G_CALLBACK(OnAbout), (GtkWidget*) pWindow);
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenu), pMenuItem);
  /* ETAPE 4 */
  pMenuItem = gtk_menu_item_new_with_label("Help");
  /* ETAPE 5 */
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(pMenuItem), pMenu);
  /* ETAPE 6 */
  gtk_menu_shell_append(GTK_MENU_SHELL(pMenuBar), pMenuItem);

  /* Ajout du menu a la fenetre */
  gtk_box_pack_start(GTK_BOX(vbox), pMenuBar, FALSE, FALSE, 0);
 

  /* Create a NoteBook */
  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), browse(), gtk_label_new (("Browse")) );
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), search(), gtk_label_new (("Search")) );
/*  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), my_tasks(), gtk_label_new (("My tasks")) );	*/
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), about(), gtk_label_new (("About")) );
  
  gtk_box_pack_start ( GTK_BOX (vbox), notebook, TRUE, TRUE, 0);

  gtk.pStatusBar = gtk_statusbar_new();
  gtk_box_pack_end(GTK_BOX(vbox), gtk.pStatusBar, FALSE, FALSE, 0);
  gtk_statusbar_push (GTK_STATUSBAR (gtk.pStatusBar), GPOINTER_TO_INT(gtk.iContextId), "No Scanning");
  
  /* Affichage de la fenetre */
  gtk_widget_show_all(pWindow);

  /* Demarrage de la boucle evenementielle */
  gtk_main();

  /* free gtk structure definitions */
  
  g_object_unref(gtk.gtk_pix_network);
  g_object_unref(gtk.gtk_pix_computer);
  g_object_unref(gtk.gtk_pix_dir);
  g_object_unref(gtk.gtk_pix_file);

  return EXIT_SUCCESS;
}



void OnAbout(GtkWidget* widget, gpointer data)
{
    GtkWidget *pAbout;

    pAbout = gtk_message_dialog_new (GTK_WINDOW(data),
        GTK_DIALOG_MODAL,
        GTK_MESSAGE_INFO,
        GTK_BUTTONS_OK,
	PROG_NAME
	" v"
	PROG_VERSION
        "\n"                             
        "Guillaume Roche\n"
        "Thanks to Aurelien Jacobs\n");

    gtk_dialog_run(GTK_DIALOG(pAbout));

    gtk_widget_destroy(pAbout);
}



void quitter(GtkWidget* widget, gpointer data)
{
    GtkWidget *pQuestion;
    /* Creation de la boite de message */
    /* Type : Question -> GTK_MESSAGE_QUESTION */
    /* Boutons : 1 OUI, 1 NON -> GTK_BUTTONS_YES_NO */
    pQuestion = gtk_message_dialog_new (GTK_WINDOW(data),
        GTK_DIALOG_MODAL,
        GTK_MESSAGE_QUESTION,
        GTK_BUTTONS_YES_NO,
        "Voulez vous vraiment\nquitter ce programme?");

    /* Affichage et attente d une reponse */
    switch(gtk_dialog_run(GTK_DIALOG(pQuestion)))
    {
        case GTK_RESPONSE_YES:
            /* OUI -> on quitte l application */
            gtk_main_quit();

            /* free things */
	    
            break;
        case GTK_RESPONSE_NO:
            /* NON -> on detruit la boite de message */
            gtk_widget_destroy(pQuestion);
            break;
    }


}


void OnDestroy(GtkWidget *pWidget, gpointer pData)
{
  /* Arret de la boucle evenementielle */
  gtk_main_quit();
  
}
