/*
  Originally developped for the SmbShareScan project.
  Copyright (C) 2004-2005  Guillaume Roche

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* system values */
#ifndef INET_ADDRSTRLEN
#define INET_ADDRSTRLEN 16
#endif

#ifndef INET6_ADDRSTRLEN
#define INET6_ADDRSTRLEN 46
#endif


/* internal values */

#define PROG_NAME "SmbShareScan"
#define PROG_VERSION  "0.6.5"

#ifndef NETWORK_PREFIX
#define NETWORK_PREFIX "192.168.1"
#endif

#define IP_TO_SCAN	1

#define URL_SIZE 	256
#define PATHLEN		256

/* debug */

#define SMB_NOT_A_COMPUTER "Computer (No name)"

/* structures */

#if 0

struct smbcfile{
  int file_smbctype;
  char *file_name;
  char *file_url;
  int file_list_size;
  int file_len;
  fileshare *file_share;	/* in which fileshare / global shared directory we are */
  
  smbcfile **file_list;
};

struct smbcdir {
  char *dir_name;
  char *dir_url;
  int file_list_size;
  int file_len;
  fileshare *file_share;
  
  smbcfile **file_list; 
};

#endif


static inline void* 
Malloc( size ) 
{
  void * ptr = malloc ( size );
  if( ptr == NULL ) {
    printf("FATAL ERROR: no more memory - cannot allocate %d bytes\n", size );
    exit (1);
  }
  
  return ptr;
}


 /* fileshare definition : files and directories */

struct fileshare {
  STAILQ_ENTRY(fileshare) share_next;

  STAILQ_HEAD(filesharelist, fileshare) share_list;
  unsigned int share_size;

  struct computer *share_computer;

  unsigned int share_type;

  const char *share_name;    /* either file or dir name */
  const char *share_url;     /* full path */
};

static inline struct fileshare *
fileshare_alloc( struct computer * comp, unsigned int type, const char * name, const char * url )
{
  struct fileshare * share = (struct fileshare*) Malloc ( sizeof( struct fileshare ));
  STAILQ_INIT( &share->share_list );

  share->share_type = type;

  share->share_computer = comp;
  share->share_name = name;
  share->share_url = url;

  return share;
}

 /* computer definition */

struct computer {
  STAILQ_ENTRY(computer) computer_next;

  STAILQ_HEAD(scomputerfilesharelist, fileshare) computer_share;
  unsigned int computer_share_size;

  const struct workgroup *computer_workgroup;
  const char *computer_netbiosname;
  const char *computer_ip;
};

static inline struct computer*
computer_alloc( const struct workgroup * workgroup, const char * netbios, const char * ip )
{
  struct computer *computer = (struct computer*) Malloc ( sizeof( struct computer ));
  STAILQ_INIT( &computer->computer_share );
  computer->computer_share_size = 0;

  computer->computer_workgroup = workgroup;
  computer->computer_netbiosname = netbios;
  computer->computer_ip = ip;

  return computer;
}

 /* workgroup definition */

struct workgroup {
  STAILQ_ENTRY(workgroup) workgroup_next;

  STAILQ_HEAD(scomputerlist, computer) workgroup_computer;
  unsigned int workgroup_size;
  
  const char *workgroup_name;
};

static inline struct workgroup*
workgroup_alloc( const char* name )
{
  struct workgroup *workgroup = (struct workgroup*) Malloc ( sizeof( struct workgroup ));
  STAILQ_INIT( &workgroup->workgroup_computer );
  workgroup->workgroup_size = 0;
  workgroup->workgroup_name = name;

  return workgroup;
}

 /* network definition */

struct network {
  STAILQ_ENTRY(network) network_next;

  STAILQ_HEAD(sworkgrouplist, workgroup) network_workgroup;
  unsigned int network_size;
};

static inline struct network*
network_alloc( void )
{
  struct network *net = (struct network*) Malloc ( sizeof( struct network ));
  STAILQ_INIT( &net->network_workgroup );
  net->network_size = 0;

  return net;
}

 /* samba */

struct samba_state {
  STAILQ_HEAD(snetworklist, network) smb_network;
  unsigned int smb_network_size;
	
  unsigned char smb_scan_state;
#define SMB_SCANNING    1
#define SMB_NO_SCAN     2
  unsigned int smb_threads;
  unsigned long smb_files;
  unsigned long smb_dirs;
  unsigned long smb_mem_used;
  char smb_default_workgroup[PATHLEN];
  char smb_default_wins_server[PATHLEN];

  pthread_mutex_t smb_network_mutex;
  pthread_mutex_t smb_thread_mutex;

  unsigned int smb_error_lvl;
};

extern struct samba_state samba;


#define SMBCCASE(x) case x: printf("\ttype: %s\n", #x ); break;

static inline int display_dir_entry ( struct smbc_dirent * dir ) {
  
  if ( samba.smb_error_lvl < 2)
    return 0;
  
  printf("Samba element : \n\tname: %s\n", dir->name );
  switch ( dir->smbc_type ) {
    SMBCCASE(SMBC_WORKGROUP)
    SMBCCASE(SMBC_SERVER)
    SMBCCASE(SMBC_FILE_SHARE)
    SMBCCASE(SMBC_PRINTER_SHARE)
    SMBCCASE(SMBC_COMMS_SHARE)
    SMBCCASE(SMBC_IPC_SHARE)
    SMBCCASE(SMBC_DIR)
    SMBCCASE(SMBC_FILE)
    SMBCCASE(SMBC_LINK)
    default:printf("unknown type\n");
  }
  return 0;
}



static inline void computer_print( const struct computer * computer )
{
  if ( samba.smb_error_lvl < 2)
    return ;	

  printf("computer: \n\tworkgroup:  %s\n\tnetbios:    %s\n\tip address: %s\n",
    computer->computer_workgroup->workgroup_name,
    computer->computer_netbiosname,
    computer->computer_ip );
}

int init_samba( void );

/* thead functions */
void* scan_ips(void *net);
void* scan_bm(void *net);

/*void search_file_in_dir(struct smbcfile *file, const char *title, smbcfile ***result, int *size);*/
