/*
  Originally developped for the SmbShareScan project.
  Copyright (C) 2004-2005  Guillaume Roche

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libsmbclient.h>
#include <errno.h>
#include <regex.h>
#include <pthread.h>
#include <unistd.h>
#include <ctype.h>
#include "bsd_sys_queue.h"
#include "samba.h"

extern struct samba_state samba;

extern int browse_fileshare( struct fileshare *);

static int browse_network_bm(struct network *net);
static int browse_workgroup_bm(struct workgroup *work);
static int browse_computer_bm(struct computer *computer);

/* SCAN WITH BROWSE MASTER */

void* scan_bm(void * arg) {
  struct network * net = (struct network*)arg;
  
  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: Browse Master Scan\n");

  browse_network_bm(net);

  if ( samba.smb_error_lvl >= 1 )
    printf ("Samba: Browse Master Scan OK\n");

  pthread_mutex_lock( &samba.smb_thread_mutex );
  samba.smb_threads--;
  pthread_mutex_unlock( &samba.smb_thread_mutex );

  pthread_exit ( NULL );
  return NULL;
}


/* static functions */

static int browse_network_bm(struct network *net) {
  struct workgroup * workgroup = NULL;
  int dh, err = 0;
  struct smbc_dirent *dirp;

  if ((dh = smbc_opendir("smb://")) < 0){
    perror("Samba: Error no response from Browse Master");
    return -1;
  }

  while ((dirp = smbc_readdir(dh))) {

    if ( samba.smb_error_lvl >= 1 )
      printf("Samba: BM: found workgroup <%s>\n", dirp->name );
	  
    if (dirp->smbc_type == 1) {

      workgroup = workgroup_alloc( strdup(dirp->name) );

      STAILQ_INSERT_TAIL( &net->network_workgroup, workgroup, workgroup_next );
      net->network_size ++;

    }
  }

  if ( errno != 0 ) {
    perror ( "Samba: problem while reading browse master directory" );
    err = 1;
  }

  err = smbc_closedir(dh);
  if ( err < 0 ) {
    perror( "Samba: problem while browsing Browse Master" );
  }

  STAILQ_FOREACH( workgroup, &net->network_workgroup, workgroup_next )
    browse_workgroup_bm(workgroup);

  return err;
}


static int browse_workgroup_bm(struct workgroup *work) {
  struct computer * computer_no_name = NULL;
  struct computer * computer = NULL;

  struct smbc_dirent *dirp = NULL;
  char url[URL_SIZE];

  int dh = 0;

  if ( samba.smb_error_lvl >= 1 )
    printf ("Samba: Browse workgroup <%s>\n", work->workgroup_name);

  snprintf(url, sizeof( url ), "smb://%s", work->workgroup_name);
  if ((dh = smbc_opendir(url)) < 0){
    if ( samba.smb_error_lvl >= 1 )
      printf("Samba: Unable to open directory : %s: %s\n", work->workgroup_name, strerror(errno));
  }

  while ((dirp = smbc_readdir(dh))) {
    /*display_dir_entry( dirp );*/

    if ( dirp->smbc_type != SMBC_SERVER && dirp->smbc_type != SMBC_FILE_SHARE )
      continue;

    if ( dirp->smbc_type == SMBC_FILE_SHARE ) {
      if ( samba.smb_error_lvl >= 1 )
        printf("Samba: in workgroup <%s> No server name\n", work->workgroup_name );

      if ( computer_no_name ) /* only one of this type allowed */
        continue;

      computer_no_name = computer_alloc ( work, 
		                          strdup(work->workgroup_name)/*strdup(SMB_NOT_A_COMPUTER)*/, 
					  strdup(work->workgroup_name) );

      STAILQ_INSERT_TAIL( &work->workgroup_computer, computer_no_name, computer_next );
      work->workgroup_size ++;

      browse_computer_bm( computer_no_name );

      continue;
    }

    computer = computer_alloc ( work, strdup(dirp->name), strdup(work->workgroup_name) );

    STAILQ_INSERT_TAIL( &work->workgroup_computer, computer, computer_next );
    work->workgroup_size ++;

    browse_computer_bm( computer );
  }

  smbc_closedir(dh);

  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: FIN Browse workgroup : %s\n", work->workgroup_name);
  return 0;
}


static int browse_computer_bm(struct computer *computer) {
  struct smbc_dirent *dir = NULL;
  struct fileshare * share = NULL;
  char url[URL_SIZE];

  int fd = 0;

  if ( samba.smb_error_lvl >= 2 ) {
    printf("Browse Computer: \n");
    computer_print( computer );
  }

  snprintf(url, sizeof(url), "smb://%s", computer->computer_netbiosname);
  
  fd = smbc_opendir(url);
  if (fd < 0) {
    if ( samba.smb_error_lvl >= 1 )
      printf("Samba: Cannot open directory <%s>: %s\n", url, strerror(errno));
    return -1;
  }

  while ((dir = smbc_readdir(fd))) {
    if (dir->smbc_type != SMBC_FILE_SHARE)
      continue;

    /* alloc a directory node */
    share = fileshare_alloc( computer, SMBC_DIR, strdup(dir->name), strdup(dir->name));

    /* insert node to computer list */
    STAILQ_INSERT_TAIL(&computer->computer_share, share, share_next);
    computer->computer_share_size ++;

    snprintf(url, sizeof(url), "%s/%s", computer->computer_ip, share->share_url);
    /* browse all sub directories */
    browse_fileshare(share);
  }

  smbc_closedir(fd);

  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: FIN Browse Computer : %s\n", computer->computer_netbiosname);

  return 0;
}



