/*
  Originally developped for the SmbShareScan project.
  Copyright (C) 2004-2005  Guillaume Roche

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libsmbclient.h>
#include <errno.h>
#include <regex.h>
#include <pthread.h>
#include <unistd.h>
#include <ctype.h>
#include "bsd_sys_queue.h"
#include "samba.h"

extern struct samba_state samba;

/* WITH IPS */

extern int browse_fileshare( struct fileshare * );

/* local functions */

static int browse_network_by_ips(struct network *net);
static int browse_workgroup_by_ips(struct workgroup *work);
static int browse_computer_by_ips(struct computer *computer, int fd);

static void* browse_workgroup_computer_by_its( void* arg );

/* thread functions */

void* scan_ips(void * arg) {
  struct network * net = (struct network*)arg;

  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: Scanning IPS\n");

  browse_network_by_ips(net);

  if ( samba.smb_error_lvl >= 1 )
    printf ("Samba: IPS Scan OK\n");

  pthread_mutex_lock( &samba.smb_thread_mutex );
  samba.smb_threads--;
  pthread_mutex_unlock( &samba.smb_thread_mutex );
  
  pthread_exit( NULL );
  
  return NULL;

}

/* local functions */

static int browse_network_by_ips( struct network *net ) {
  struct workgroup * workgroup = NULL;

  if ( samba.smb_error_lvl >= 1 )
    printf ("Samba: Browse network by IPS\n");

  workgroup = workgroup_alloc ( strdup( "Network (no name)" ) );
  STAILQ_INSERT_TAIL( &net->network_workgroup, workgroup, workgroup_next );
  net->network_size ++;

  browse_workgroup_by_ips( workgroup );

  return 0;
}

struct ips_thread_data {
	char data_url[URL_SIZE];
	char data_address[INET_ADDRSTRLEN];
	struct workgroup * data_workgroup;
};

static int browse_workgroup_by_ips(struct workgroup *work) {
  pthread_t process[255];
  void* returned_value;

  int j = 0, err;

  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: workgroup search <%s> by ips ...\n", work->workgroup_name );

  for(j=1; j<255; j++) {
    struct ips_thread_data * data = Malloc( sizeof( struct ips_thread_data ));

    snprintf(data->data_address, sizeof(data->data_address), "%s.%d", NETWORK_PREFIX, j);
    snprintf(data->data_url, sizeof(data->data_url), "smb://%s", data->data_address);
    data->data_workgroup = work;

    usleep( 200000 );

    err = pthread_create (&process[j], NULL, browse_workgroup_computer_by_its, data );
    if ( err < 0 ) {
      fprintf (stderr, "Impossible de crer un thread\n");
      exit (1);
    } else {
      pthread_mutex_lock( &samba.smb_thread_mutex );
      samba.smb_threads++;
      pthread_mutex_unlock( &samba.smb_thread_mutex );
    }
  }

  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: workgroup search <%s> : all scans started\n", work->workgroup_name );

  for(j=1; j<255; j++) {
    pthread_join ( process[j], &returned_value );
  }

  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: workgroup search <%s> finished\n", work->workgroup_name);

  return 0;
}


static void* browse_workgroup_computer_by_its( void* arg )
{
    struct ips_thread_data * data = (struct ips_thread_data *) arg;
    struct computer * computer = NULL;
    int dh = 0;

    const char * address = data->data_address;
    const char * url = data->data_url;
    struct workgroup * work = data->data_workgroup;
    
    if ( samba.smb_error_lvl >= 1 )
      printf("Samba: workgroup <%s> URL %s\n", work->workgroup_name, url);

    if ((dh = smbc_opendir(url)) < 0) {
      if ( samba.smb_error_lvl >= 1 )
        printf("Samba: Unable to open workgroup <%s> URL %s\n", work->workgroup_name, url);
      free( data );
      
      pthread_mutex_lock( &samba.smb_thread_mutex );
      samba.smb_threads--;
      pthread_mutex_unlock( &samba.smb_thread_mutex );

      pthread_exit( NULL );
    }

    /* there is a computer, we have to lock all things to add the computer's data */
    pthread_mutex_lock( &samba.smb_network_mutex );

    /* insert entry */
    computer = computer_alloc( work, strdup(address), strdup(address) );
    STAILQ_INSERT_TAIL( &work->workgroup_computer, computer, computer_next );
    work->workgroup_size ++;

    /* browse computer */
    browse_computer_by_ips(computer, dh);

    pthread_mutex_unlock( &samba.smb_network_mutex );

    smbc_closedir( dh );

    pthread_mutex_lock( &samba.smb_thread_mutex );
    samba.smb_threads--;
    pthread_mutex_unlock( &samba.smb_thread_mutex );

    free( data );
    pthread_exit( NULL );
    return NULL;
}


static int browse_computer_by_ips(struct computer *computer, int computer_connection) {
  struct smbc_dirent *dir = NULL;
  struct fileshare * share = NULL;
  char url[URL_SIZE];

  if ( samba.smb_error_lvl >= 1 )
    printf("Samba: Browse Computer by IPS : %s\n", computer->computer_netbiosname);

  computer_print( computer );

  while ((dir = smbc_readdir(computer_connection))) {
    if (dir->smbc_type != SMBC_FILE_SHARE)
      continue;

    /* alloc a directory node */
    share = fileshare_alloc( computer, SMBC_DIR, strdup(dir->name), strdup(dir->name));

    /* insert node to computer list */
    STAILQ_INSERT_TAIL( &computer->computer_share, share, share_next );
    computer->computer_share_size ++;
    
    /* browse all sub directories */
    browse_fileshare(share);
  }

  if ( samba.smb_error_lvl >= 1 ) 
    printf("Samba: END IPS Browse Computer : %s\n", computer->computer_netbiosname);
  return 0;
}  



